@@ -1,15 +1,31 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+#
+# Copyright 2022 Greg Albrecht <oss@undef.net>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# Author:: Greg Albrecht W2GMD <oss@undef.net>
+#
 
-"""ADS-B Cursor-On-Target Class Definitions."""
-
+"""ADSBCOT Class Definitions."""
 
 import asyncio
-import logging
+
+from typing import Union
+from urllib.parse import ParseResult, urlparse
 
 import aiohttp
 import pytak
-
+import aircot
 import adsbcot
 
 # We won't use pyModeS if it isn't installed:
@@ -26,107 +42,135 @@ __copyright__ = "Copyright 2022 Greg Albrecht"
 __license__ = "Apache License, Version 2.0"
 
 
-class ADSBWorker(pytak.MessageWorker):
+class ADSBWorker(pytak.QueueWorker):
 
-    """Reads ADS-B Data from inputs, renders to CoT, and puts on queue."""
+    """Reads ADS-B Data from inputs, serializes to COT, and puts on TX queue."""
 
-    def __init__(  # pylint: disable=too-many-arguments
-        self,
-        event_queue,
-        url,
-        cot_stale,
-        poll_interval: int = None,
-        filters: dict = None,
-    ):
-        super().__init__(event_queue, url)
-        self.poll_interval: int = int(poll_interval or adsbcot.DEFAULT_POLL_INTERVAL)
-        self.url = url
-        self.cot_stale = cot_stale
-        self.filters = filters
-        self._logger.debug("cot_stale='%s'", self.cot_stale)
+    def __init__(self, queue, config):
+        super().__init__(queue, config)
+        self.known_craft_db = None
+        self.session = None
+        self.uid_key: str = self.config.get("UID_KEY", "ICAO")
 
-    async def handle_message(self, aircraft: list) -> None:
+        known_craft = self.config.get("KNOWN_CRAFT")
+        if known_craft:
+            self._logger.info("Using KNOWN_CRAFT: %s", known_craft)
+            self.known_craft_db = aircot.read_known_craft(known_craft)
+
+    async def handle_data(self, data: list) -> None:
         """
-        Transforms Aircraft AIS data to CoT and puts it onto tx queue.
-        """
-        if not isinstance(aircraft, list):
-            self._logger.warning("Invalid aircraft data, should be a Python list.")
-            return False
+        Handle Data from ADS-B receiver: Render to COT, put on TX queue.
 
-        if not aircraft:
+        Parameters
+        ----------
+        data : `list[dict, ]`
+            List of craft data as key/value arrays.
+        """
+        if not isinstance(data, list):
+            self._logger.warning("Invalid aircraft data, should be a Python `list`.")
+            return
+
+        if not data:
             self._logger.warning("Empty aircraft list")
-            return False
+            return
 
+        lod = len(data)
         i = 1
-        for craft in aircraft:
-            self._logger.debug("craft='%s'", craft)
-
-            if self.filters:
-                if "ICAO" in self.filters:
-                    icao = craft.get("hex", "").upper()
-                    if icao not in self.filters.get("ICAO", "include"):
-                        continue
-                    if icao in self.filters.get("ICAO", "exclude"):
-                        continue
-                elif "FLIGHT" in self.filters:
-                    flight = craft.get("flight", "").upper()
-                    if flight not in self.filters.get("FLIGHT", "include"):
-                        continue
-                    if flight in self.filters.get("FLIGHT", "exclude"):
-                        continue
-
-            event = adsbcot.adsb_to_cot(craft, stale=self.cot_stale)
-            if not event:
-                self._logger.debug("Empty CoT Event for craft=%s", craft)
-                i += 1
+        for craft in data:
+            i += 1
+            if not isinstance(craft, dict):
+                self._logger.warning("Aircraft list item was not a Python `dict`.")
                 continue
 
-            self._logger.debug(
-                "Handling %s/%s ICAO: %s Flight: %s Category: %s Registration: %s",
-                i,
-                len(aircraft),
-                craft.get("hex"),
-                craft.get("flight"),
-                craft.get("category"),
-                craft.get("reg"),
-            )
-            await self._put_event_queue(event)
-            i += 1
+            icao: str = craft.get("hex", "")
+            if icao:
+                icao = icao.strip().upper()
+            else:
+                continue
 
-    async def _get_dump1090_feed(self):
+            if "~" in icao and not self.config.getboolean("INCLUDE_TISB"):
+                continue
+
+            known_craft: dict = aircot.get_known_craft(self.known_craft_db, icao, "HEX")
+
+            # Skip if we're using known_craft CSV and this Craft isn't found:
+            if (
+                self.known_craft_db
+                and not known_craft
+                and not self.config.getboolean("INCLUDE_ALL_CRAFT")
+            ):
+                continue
+
+            event: Union[str, None] = adsbcot.adsb_to_cot(
+                craft, self.config, known_craft
+            )
+
+            if not event:
+                self._logger.debug("Empty COT Event for craft=%s", craft)
+                continue
+
+            self._logger.debug("Handling %s/%s ICAO: %s", i, lod, icao)
+            await self.put_queue(event)
+
+    async def get_dump1090_feed(self, url: str):
         """
-        Polls the dump1090 JSON API and passes data to message handler.
+        Polls the dump1090 JSON API and passes data to data handler.
         """
-        async with aiohttp.ClientSession() as session:
-            resp = await session.request(method="GET", url=self.url.geturl())
-            resp.raise_for_status()
+        async with self.session.get(url) as resp:
+            if resp.status != 200:
+                response_content = await resp.text()
+                self._logger.error("Received HTTP Status %s for %s", resp.status, url)
+                self._logger.error(response_content)
+                return
 
             json_resp = await resp.json()
-            aircraft = json_resp.get("aircraft")
-            self._logger.debug(
-                "Retrieved %s aircraft from %s", len(aircraft), self.url.geturl()
-            )
+            if json_resp == None:
+                return
 
-            await self.handle_message(aircraft)
+            data = json_resp.get("aircraft")
+            if data == None:
+                return
+
+            self._logger.info("Retrieved %s aircraft messages.", str(len(data) or "No"))
+            await self.handle_data(data)
 
     async def run(self, number_of_iterations=-1):
         """Runs this Thread, Reads from Pollers."""
-        self._logger.info("Running ADSBWorker with URL '%s'", self.url.geturl())
+        url: str = self.config.get("DUMP1090_URL", "")
+        if not url:
+            raise Exception("Please specify a DUMP1090_URL.")
 
-        while 1:
-            await self._get_dump1090_feed()
-            await asyncio.sleep(self.poll_interval)
+        self._logger.info("Running %s", self.__class__)
+
+        known_craft: str = self.config.get("KNOWN_CRAFT", "")
+        poll_interval: str = self.config.get(
+            "POLL_INTERVAL", adsbcot.DEFAULT_POLL_INTERVAL
+        )
+
+        if known_craft:
+            self._logger.info("Using KNOWN_CRAFT: %s", known_craft)
+            self.known_craft_db = aircot.read_known_craft(known_craft)
+
+        async with aiohttp.ClientSession() as self.session:
+            while 1:
+                self._logger.info(
+                    "%s polling every %ss: %s", self.__class__, poll_interval, url
+                )
+                await self.get_dump1090_feed(url)
+                await asyncio.sleep(int(poll_interval))
 
 
 class ADSBNetWorker(ADSBWorker):
     """Reads ADS-B Data from network, renders to COT, and puts on queue."""
 
     def __init__(
-        self, event_queue, net_queue, data_type, cot_stale, filters
+        self, queue, net_queue, config, data_type
     ):  # NOQA pylint: disable=too-many-arguments
-        super().__init__(event_queue, None, cot_stale, None, filters)
+        super().__init__(queue, config)
         self.net_queue = net_queue
+        self.config = config
         self.data_type = data_type
+
         self.local_buffer_adsb_msg = []
         self.local_buffer_adsb_ts = []
         self.local_buffer_commb_msg = []
@@ -143,7 +187,9 @@ class ADSBNetWorker(ADSBWorker):
         self, number_of_iterations=-1
     ):  # NOQA pylint: disable=too-many-locals, too-many-branches
         """Runs the main process loop."""
-        self._logger.info("Running ADSBNetWorker for data_type='%s'", self.data_type)
+        self._logger.info(
+            "Running %s for data_type: %s", self.__class__, self.data_type
+        )
 
         self._reset_local_buffer()
 
@@ -212,8 +258,11 @@ class ADSBNetWorker(ADSBWorker):
                 gnds = val.get("gs")
                 reg = val.get("r")
                 trk = val.get("trk")
-                if lat and lon and flight and alt_geom and gnds and trk:
-                    aircraft = [
+                # FIXME: Convert this to a filter()
+                if (  # pylint: disable=too-many-boolean-expressions
+                    lat and lon and flight and alt_geom and gnds and trk
+                ):
+                    data = [
                         {
                             "hex": key,
                             "lat": lat,
@@ -225,37 +274,28 @@ class ADSBNetWorker(ADSBWorker):
                             "trk": trk,
                         }
                     ]
-                    await self.handle_message(aircraft)
+                    await self.handle_data(data)
                 else:
                     continue
 
 
-class ADSBNetReceiver:  # pylint: disable=too-few-public-methods
+class ADSBNetReceiver(pytak.QueueWorker):  # pylint: disable=too-few-public-methods
     """Reads ADS-B Data from network and puts on queue."""
 
-    _logger = logging.getLogger(__name__)
-    if not _logger.handlers:
-        _logger.setLevel(adsbcot.LOG_LEVEL)
-        _console_handler = logging.StreamHandler()
-        _console_handler.setLevel(adsbcot.LOG_LEVEL)
-        _console_handler.setFormatter(adsbcot.LOG_FORMAT)
-        _logger.addHandler(_console_handler)
-        _logger.propagate = False
-    logging.getLogger("asyncio").setLevel(adsbcot.LOG_LEVEL)
+    def __init__(self, queue, config, data_type):
+        super().__init__(queue, config)
+        self.data_type: str = data_type
 
-    def __init__(self, net_queue, url, data_type):
-        self.net_queue = net_queue
-        self.url = url
-        self.data_type = data_type
-
-    async def run(self):
+    async def run(self, number_of_iterations=-1):
         """Runs the main process loop."""
-        self._logger.info("Running ADSBNetReceiver for URL=%s", self.url.geturl())
+        url: ParseResult = urlparse(self.config.get("DUMP1090_URL"))
 
-        if ":" in self.url.path:
-            host, port = self.url.path.split(":")
+        self._logger.info("Running %s for %s", self.__class__, url.geturl())
+
+        if ":" in url.netloc:
+            host, port = url.netloc.split(":")
         else:
-            host = self.url.path
+            host = url.netloc
             if self.data_type == "raw":
                 port = adsbcot.DEFAULT_DUMP1090_TCP_RAW_PORT
             elif self.data_type == "beast":
@@ -270,8 +310,8 @@ class ADSBNetReceiver:  # pylint: disable=too-few-public-methods
         if self.data_type == "raw":
             while 1:
                 received = await reader.readline()
-                self.net_queue.put_nowait(received)
+                self.queue.put_nowait(received)
         elif self.data_type == "beast":
             while 1:
                 received = await reader.read(4096)
-                self.net_queue.put_nowait(received)
+                self.queue.put_nowait(received)
